@@ -1,7 +1,7 @@
 /*
 	frame: Heap of routines dealing with the core mpg123 data structure.
 
-	copyright 2008 by the mpg123 project - free software under the terms of the LGPL 2.1
+	copyright 2008-9 by the mpg123 project - free software under the terms of the LGPL 2.1
 	see COPYING and AUTHORS files in distribution or http://mpg123.org
 	initially written by Thomas Orgis
 */
@@ -22,19 +22,23 @@ static void frame_fixed_reset(mpg123_handle *fr);
 	: (type*)(p)
 void frame_default_pars(mpg123_pars *mp)
 {
-	mp->outscale = MAXOUTBURST;
+	mp->outscale = 1.0;
 #ifdef GAPLESS
 	mp->flags = MPG123_GAPLESS;
 #else
 	mp->flags = 0;
 #endif
+#ifndef NO_NTOM
 	mp->force_rate = 0;
+#endif
 	mp->down_sample = 0;
 	mp->rva = 0;
 	mp->halfspeed = 0;
 	mp->doublespeed = 0;
 	mp->verbose = 0;
+#ifndef NO_ICY
 	mp->icy_interval = 0;
+#endif
 #ifndef WIN32
 	mp->timeout = 0;
 #endif
@@ -55,15 +59,21 @@ void frame_init_par(mpg123_handle *fr, mpg123_pars *mp)
 	fr->own_buffer = FALSE;
 	fr->buffer.data = NULL;
 	fr->rawbuffs = NULL;
+	fr->rawbuffss = 0;
 	fr->rawdecwin = NULL;
+	fr->rawdecwins = 0;
+#ifndef NO_8BIT
 	fr->conv16to8_buf = NULL;
+#endif
 	fr->xing_toc = NULL;
-	fr->cpu_opts.type = defopt;
-	fr->cpu_opts.class = (defopt == mmx || defopt == sse || defopt == dreidnowext) ? mmxsse : normal;
+	fr->cpu_opts.type = defdec();
+	fr->cpu_opts.class = decclass(fr->cpu_opts.type);
+#ifndef NO_NTOM
 	/* these two look unnecessary, check guarantee for synth_ntom_set_step (in control_generic, even)! */
 	fr->ntom_val[0] = NTOM_MUL>>1;
 	fr->ntom_val[1] = NTOM_MUL>>1;
 	fr->ntom_step = NTOM_MUL;
+#endif
 	/* unnecessary: fr->buffer.size = fr->buffer.fill = 0; */
 	mpg123_reset_eq(fr);
 	init_icy(&fr->icy);
@@ -81,10 +91,17 @@ void frame_init_par(mpg123_handle *fr, mpg123_pars *mp)
 
 	fr->down_sample = 0; /* Initialize to silence harmless errors when debugging. */
 	frame_fixed_reset(fr); /* Reset only the fixed data, dynamic buffers are not there yet! */
+	fr->synth = NULL;
+	fr->synth_mono = NULL;
+	fr->make_decode_tables = NULL;
 #ifdef FRAME_INDEX
 	fi_init(&fr->index);
 	frame_index_setup(fr); /* Apply the size setting. */
 #endif
+#ifdef OPT_DITHER
+	/* The idea is to read that at runtime in the future... to avoid bloating the binary. */
+	fr->dithernoise = dithernoise;
+#endif
 }
 
 mpg123_pars attribute_align_arg *mpg123_new_pars(int *error)
@@ -170,6 +187,11 @@ int frame_index_setup(mpg123_handle *fr)
 }
 #endif
 
+static void frame_decode_buffers_reset(mpg123_handle *fr)
+{
+	memset(fr->rawbuffs, 0, fr->rawbuffss);
+}
+
 int frame_buffers(mpg123_handle *fr)
 {
 	int buffssize = 0;
@@ -239,7 +261,6 @@ int frame_buffers(mpg123_handle *fr)
 	/* The MMX ones want 32byte alignment, which I'll try to ensure manually */
 	{
 		int decwin_size = (512+32)*sizeof(real);
-		if(fr->rawdecwin != NULL) free(fr->rawdecwin);
 #ifdef OPT_MMXORSSE
 #ifdef OPT_MULTI
 		if(fr->cpu_opts.class == mmxsse)
@@ -253,8 +274,19 @@ int frame_buffers(mpg123_handle *fr)
 		}
 #endif
 #endif
+		/* Hm, that's basically realloc() ... */
+		if(fr->rawdecwin != NULL && fr->rawdecwins != decwin_size)
+		{
+			free(fr->rawdecwin);
+			fr->rawdecwin = NULL;
+		}
+
+		if(fr->rawdecwin == NULL)
 		fr->rawdecwin = (unsigned char*) malloc(decwin_size);
+
 		if(fr->rawdecwin == NULL) return -1;
+
+		fr->rawdecwins = decwin_size;
 		fr->decwin = (real*) fr->rawdecwin;
 #ifdef OPT_MMXORSSE
 #ifdef OPT_MULTI
@@ -273,7 +305,9 @@ int frame_buffers(mpg123_handle *fr)
 #endif
 #endif
 	}
-	frame_buffers_reset(fr);
+	/* Only reset the buffers we created just now. */
+	frame_decode_buffers_reset(fr);
+
 	debug1("frame %p buffer done", (void*)fr);
 	return 0;
 }
@@ -286,21 +320,22 @@ int frame_buffers_reset(mpg123_handle *fr)
 	fr->bsbuf = fr->bsspace[1];
 	fr->bsbufold = fr->bsbuf;
 	fr->bitreservoir = 0; /* Not entirely sure if this is the right place for that counter. */
+	frame_decode_buffers_reset(fr);
 	memset(fr->bsspace, 0, 2*(MAXFRAMESIZE+512));
 	memset(fr->ssave, 0, 34);
-	memset(fr->rawbuffs, 0, fr->rawbuffss);
 	fr->hybrid_blc[0] = fr->hybrid_blc[1] = 0;
 	memset(fr->hybrid_block, 0, sizeof(real)*2*2*SBLIMIT*SSLIMIT);
-	/* Not totally, but quite, sure that decwin(s) doesn't need cleaning. */
 	return 0;
 }
 
 void frame_icy_reset(mpg123_handle* fr)
 {
+#ifndef NO_ICY
 	if(fr->icy.data != NULL) free(fr->icy.data);
 	fr->icy.data = NULL;
 	fr->icy.interval = 0;
 	fr->icy.next = 0;
+#endif
 }
 
 void frame_free_toc(mpg123_handle *fr)
@@ -352,6 +387,7 @@ static void frame_fixed_reset(mpg123_handle *fr)
 	fr->metaflags = 0;
 	fr->outblock = mpg123_safe_buffer();
 	fr->num = -1;
+	fr->playnum = -1;
 	fr->accurate = TRUE;
 	fr->silent_resync = 0;
 	fr->audio_start = 0;
@@ -384,27 +420,36 @@ static void frame_fixed_reset(mpg123_handle *fr)
 	fr->lastoff = 0;
 	fr->firstoff = 0;
 #endif
-	fr->bo[0] = 1; /* the usual bo */
-	fr->bo[1] = 0; /* ditherindex */
 #ifdef OPT_I486
-	fr->bo[0] = fr->bo[1] = FIR_SIZE-1;
+	fr->i486bo[0] = fr->i486bo[1] = FIR_SIZE-1;
+#endif
+	fr->bo = 1; /* the usual bo */
+#ifdef OPT_DITHER
+	fr->ditherindex = 0;
 #endif
 	reset_id3(fr);
 	reset_icy(&fr->icy);
 	/* ICY stuff should go into icy.c, eh? */
+#ifndef NO_ICY
 	fr->icy.interval = 0;
 	fr->icy.next = 0;
+#endif
 	fr->halfphase = 0; /* here or indeed only on first-time init? */
+	fr->error_protection = 0;
 }
 
 void frame_free_buffers(mpg123_handle *fr)
 {
 	if(fr->rawbuffs != NULL) free(fr->rawbuffs);
 	fr->rawbuffs = NULL;
+	fr->rawbuffss = 0;
 	if(fr->rawdecwin != NULL) free(fr->rawdecwin);
 	fr->rawdecwin = NULL;
+	fr->rawdecwins = 0;
+#ifndef NO_8BIT
 	if(fr->conv16to8_buf != NULL) free(fr->conv16to8_buf);
 	fr->conv16to8_buf = NULL;
+#endif
 }
 
 void frame_exit(mpg123_handle *fr)
@@ -565,9 +610,15 @@ off_t frame_ins2outs(mpg123_handle *fr, off_t ins)
 	switch(fr->down_sample)
 	{
 		case 0:
+#		ifndef NO_DOWNSAMPLE
 		case 1:
-		case 2: outs = ins>>fr->down_sample; break;
+		case 2:
+#		endif
+			outs = ins>>fr->down_sample;
+		break;
+#		ifndef NO_NTOM
 		case 3: outs = ntom_ins2outs(fr, ins); break;
+#		endif
 		default: error1("Bad down_sample (%i) ... should not be possible!!", fr->down_sample);
 	}
 	return outs;
@@ -579,9 +630,15 @@ off_t frame_outs(mpg123_handle *fr, off_t num)
 	switch(fr->down_sample)
 	{
 		case 0:
+#		ifndef NO_DOWNSAMPLE
 		case 1:
-		case 2: outs = (spf(fr)>>fr->down_sample)*num; break;
+		case 2:
+#		endif
+			outs = (spf(fr)>>fr->down_sample)*num;
+		break;
+#ifndef NO_NTOM
 		case 3: outs = ntom_frmouts(fr, num); break;
+#endif
 		default: error1("Bad down_sample (%i) ... should not be possible!!", fr->down_sample);
 	}
 	return outs;
@@ -593,9 +650,15 @@ off_t frame_offset(mpg123_handle *fr, off_t outs)
 	switch(fr->down_sample)
 	{
 		case 0:
+#		ifndef NO_DOWNSAMPLE
 		case 1:
-		case 2: num = outs/(spf(fr)>>fr->down_sample); break;
+		case 2:
+#		endif
+			num = outs/(spf(fr)>>fr->down_sample);
+		break;
+#ifndef NO_NTOM
 		case 3: num = ntom_frameoff(fr, outs); break;
+#endif
 		default: error("Bad down_sample ... should not be possible!!");
 	}
 	return num;
@@ -658,11 +721,21 @@ void frame_set_frameseek(mpg123_handle *fr, off_t fe)
 #endif
 }
 
+void frame_skip(mpg123_handle *fr)
+{
+#ifndef NO_LAYER3
+	if(fr->lay == 3) set_pointer(fr, 512);
+#endif
+}
+
 /* Sample accurate seek prepare for decoder. */
 /* This gets unadjusted output samples and takes resampling into account */
 void frame_set_seek(mpg123_handle *fr, off_t sp)
 {
 	fr->firstframe = frame_offset(fr, sp);
+#ifndef NO_NTOM
+	if(fr->down_sample == 3) ntom_set_ntom(fr, fr->firstframe);
+#endif
 	fr->ignoreframe = fr->lay == 3 ? fr->firstframe-IGNORESHIFT : fr->firstframe;
 #ifdef GAPLESS /* The sample offset is used for non-gapless mode, too! */
 	fr->firstoff = sp - frame_outs(fr, fr->firstframe);
@@ -675,81 +748,19 @@ void frame_set_seek(mpg123_handle *fr, off_t sp)
 #endif
 }
 
-/* to vanish */
-void frame_outformat(mpg123_handle *fr, int format, int channels, long rate)
-{
-	fr->af.encoding = format;
-	fr->af.rate = rate;
-	fr->af.channels = channels;
-}
-
-/* set synth functions for current frame, optimizations handled by opt_* macros */
-int set_synth_functions(mpg123_handle *fr)
-{
-	int ds = fr->down_sample;
-	int p8=0;
-	static func_synth funcs[2][4] = { 
-		{ NULL,
-		  synth_2to1,
-		  synth_4to1,
-		  synth_ntom } ,
-		{ NULL,
-		  synth_2to1_8bit,
-		  synth_4to1_8bit,
-		  synth_ntom_8bit } 
-	};
-	static func_synth_mono funcs_mono[2][2][4] = {    
-		{ { NULL ,
-		    synth_2to1_mono2stereo ,
-		    synth_4to1_mono2stereo ,
-		    synth_ntom_mono2stereo } ,
-		  { NULL ,
-		    synth_2to1_8bit_mono2stereo ,
-		    synth_4to1_8bit_mono2stereo ,
-		    synth_ntom_8bit_mono2stereo } } ,
-		{ { NULL ,
-		    synth_2to1_mono ,
-		    synth_4to1_mono ,
-		    synth_ntom_mono } ,
-		  { NULL ,
-		    synth_2to1_8bit_mono ,
-		    synth_4to1_8bit_mono ,
-		    synth_ntom_8bit_mono } }
-	};
-
-	/* possibly non-constand entries filled here */
-	funcs[0][0] = (func_synth) opt_synth_1to1(fr);
-	funcs[1][0] = (func_synth) opt_synth_1to1_8bit(fr);
-	funcs_mono[0][0][0] = (func_synth_mono) opt_synth_1to1_mono2stereo(fr);
-	funcs_mono[0][1][0] = (func_synth_mono) opt_synth_1to1_8bit_mono2stereo(fr);
-	funcs_mono[1][0][0] = (func_synth_mono) opt_synth_1to1_mono(fr);
-	funcs_mono[1][1][0] = (func_synth_mono) opt_synth_1to1_8bit_mono(fr);
-
-	if(fr->af.encoding & MPG123_ENC_8) p8 = 1;
-	fr->synth = funcs[p8][ds];
-	fr->synth_mono = funcs_mono[fr->af.channels==2 ? 0 : 1][p8][ds];
-
-	if(p8)
-	{
-		if(make_conv16to8_table(fr) != 0)
-		{
-			/* it's a bit more work to get proper error propagation up */
-			return -1;
-		}
-	}
-	return 0;
-}
-
 int attribute_align_arg mpg123_volume_change(mpg123_handle *mh, double change)
 {
 	if(mh == NULL) return MPG123_ERR;
-	return mpg123_volume(mh, change + (double) mh->p.outscale / MAXOUTBURST);
+	return mpg123_volume(mh, change + (double) mh->p.outscale);
 }
 
 int attribute_align_arg mpg123_volume(mpg123_handle *mh, double vol)
 {
 	if(mh == NULL) return MPG123_ERR;
-	if(vol >= 0) mh->p.outscale = (double) MAXOUTBURST * vol;
+
+	if(vol >= 0) mh->p.outscale = vol;
+	else mh->p.outscale = 0.;
+
 	do_rva(mh);
 	return MPG123_OK;
 }
@@ -781,7 +792,7 @@ void do_rva(mpg123_handle *fr)
 {
 	double peak = 0;
 	double gain = 0;
-	scale_t newscale;
+	double newscale;
 	double rvafact = 1;
 	if(get_rva(fr, &peak, &gain))
 	{
@@ -792,294 +803,28 @@ void do_rva(mpg123_handle *fr)
 	newscale = fr->p.outscale*rvafact;
 
 	/* if peak is unknown (== 0) this check won't hurt */
-	if((peak*newscale) > MAXOUTBURST)
+	if((peak*newscale) > 1.0)
 	{
-		newscale = (scale_t) ((double) MAXOUTBURST/peak);
-#ifdef FLOATOUT
+		newscale = 1.0/peak;
 		warning2("limiting scale value to %f to prevent clipping with indicated peak factor of %f", newscale, peak);
-#else
-		warning2("limiting scale value to %li to prevent clipping with indicated peak factor of %f", newscale, peak);
-#endif
 	}
 	/* first rva setting is forced with fr->lastscale < 0 */
-	if(newscale != fr->lastscale)
+	if(newscale != fr->lastscale || fr->decoder_change)
 	{
-#ifdef FLOATOUT
 		debug3("changing scale value from %f to %f (peak estimated to %f)", fr->lastscale != -1 ? fr->lastscale : fr->p.outscale, newscale, (double) (newscale*peak));
-#else
-		debug3("changing scale value from %li to %li (peak estimated to %li)", fr->lastscale != -1 ? fr->lastscale : fr->p.outscale, newscale, (long) (newscale*peak));
-#endif
 		fr->lastscale = newscale;
-		opt_make_decode_tables(fr); /* the actual work */
+		/* It may be too early, actually. */
+		if(fr->make_decode_tables != NULL) fr->make_decode_tables(fr); /* the actual work */
 	}
 }
 
+
 int attribute_align_arg mpg123_getvolume(mpg123_handle *mh, double *base, double *really, double *rva_db)
 {
 	if(mh == NULL) return MPG123_ERR;
-	if(base)   *base   = (double)mh->p.outscale/MAXOUTBURST;
-	if(really) *really = (double)mh->lastscale/MAXOUTBURST;
+	if(base)   *base   = mh->p.outscale;
+	if(really) *really = mh->lastscale;
 	get_rva(mh, NULL, rva_db);
 	return MPG123_OK;
 }
 
-int  frame_cpu_opt(mpg123_handle *fr, const char* cpu)
-{
-	char* chosen = ""; /* the chosed decoder opt as string */
-	int auto_choose = 0;
-	int done = 0;
-	if(   (cpu == NULL)
-	   || (cpu[0] == 0)
-	   || !strcasecmp(cpu, "auto") )
-	auto_choose = 1;
-#ifndef OPT_MULTI
-	{
-		char **sd = mpg123_decoders(); /* this contains _one_ decoder */
-		if(!auto_choose && strcasecmp(cpu, sd[0])) done = 0;
-		else
-		{
-			chosen = sd[0];
-			done = 1;
-		}
-	}
-#else
-	fr->cpu_opts.type = nodec;
-	/* covers any i386+ cpu; they actually differ only in the synth_1to1 function... */
-	#ifdef OPT_X86
-
-	#ifdef OPT_MMXORSSE
-	fr->cpu_opts.make_decode_tables   = make_decode_tables;
-	fr->cpu_opts.init_layer3_gainpow2 = init_layer3_gainpow2;
-	fr->cpu_opts.init_layer2_table    = init_layer2_table;
-	#endif
-	#ifdef OPT_3DNOW
-	fr->cpu_opts.dct36 = dct36;
-	#endif
-	#ifdef OPT_3DNOWEXT
-	fr->cpu_opts.dct36 = dct36;
-	#endif
-
-	if(cpu_i586(cpu_flags))
-	{
-		debug2("standard flags: 0x%08x\textended flags: 0x%08x", cpu_flags.std, cpu_flags.ext);
-		#ifdef OPT_3DNOWEXT
-		if(   !done && (auto_choose || !strcasecmp(cpu, "3dnowext"))
-		   && cpu_3dnow(cpu_flags)
-		   && cpu_3dnowext(cpu_flags)
-		   && cpu_mmx(cpu_flags) )
-		{
-			int go = 1;
-			if(fr->p.force_rate)
-			{
-				#if defined(K6_FALLBACK) || defined(PENTIUM_FALLBACK)
-				if(!auto_choose){ if(NOQUIET) error("I refuse to choose 3DNowExt as this will screw up with forced rate!"); }
-				else if(VERBOSE) fprintf(stderr, "Note: Not choosing 3DNowExt because flexible rate not supported.\n");
-
-				go = 0;
-				#else
-				if(NOQUIET) error("You will hear some awful sound because of flexible rate being chosen with 3DNowExt decoder!");
-				#endif
-			}
-			if(go){ /* temporary hack for flexible rate bug, not going indent this - fix it instead! */
-			chosen = "3DNowExt";
-			fr->cpu_opts.type = dreidnowext;
-			fr->cpu_opts.class = mmxsse;
-			fr->cpu_opts.dct36 = dct36_3dnowext;
-			fr->cpu_opts.synth_1to1 = synth_1to1_3dnowext;
-			fr->cpu_opts.dct64 = dct64_mmx; /* only use the 3dnow version in the synth_1to1_sse */
-			fr->cpu_opts.make_decode_tables   = make_decode_tables_mmx;
-			fr->cpu_opts.init_layer3_gainpow2 = init_layer3_gainpow2_mmx;
-			fr->cpu_opts.init_layer2_table    = init_layer2_table_mmx;
-			fr->cpu_opts.mpl_dct64 = dct64_3dnowext;
-			done = 1;
-			}
-		}
-		#endif
-		#ifdef OPT_SSE
-		if(   !done && (auto_choose || !strcasecmp(cpu, "sse"))
-		   && cpu_sse(cpu_flags) && cpu_mmx(cpu_flags) )
-		{
-			int go = 1;
-			if(fr->p.force_rate)
-			{
-				#ifdef PENTIUM_FALLBACK
-				if(!auto_choose){ if(NOQUIET) error("I refuse to choose SSE as this will screw up with forced rate!"); }
-				else if(VERBOSE) fprintf(stderr, "Note: Not choosing SSE because flexible rate not supported.\n");
-
-				go = 0;
-				#else
-				if(NOQUIET) error("You will hear some awful sound because of flexible rate being chosen with SSE decoder!");
-				#endif
-			}
-			if(go){ /* temporary hack for flexible rate bug, not going indent this - fix it instead! */
-			chosen = "SSE";
-			fr->cpu_opts.type = sse;
-			fr->cpu_opts.class = mmxsse;
-			fr->cpu_opts.synth_1to1 = synth_1to1_sse;
-			fr->cpu_opts.dct64 = dct64_mmx; /* only use the sse version in the synth_1to1_sse */
-			fr->cpu_opts.make_decode_tables   = make_decode_tables_mmx;
-			fr->cpu_opts.init_layer3_gainpow2 = init_layer3_gainpow2_mmx;
-			fr->cpu_opts.init_layer2_table    = init_layer2_table_mmx;
-			fr->cpu_opts.mpl_dct64 = dct64_sse;
-			done = 1;
-			}
-		}
-		#endif
-		#ifdef OPT_3DNOW
-		fr->cpu_opts.dct36 = dct36;
-		/* TODO: make autodetection for _all_ x86 optimizations (maybe just for i586+ and keep separate 486 build?) */
-		/* check cpuflags bit 31 (3DNow!) and 23 (MMX) */
-		if(    !done && (auto_choose || !strcasecmp(cpu, "3dnow"))
-		    && cpu_3dnow(cpu_flags) && cpu_mmx(cpu_flags) )
-		{
-			chosen = "3DNow";
-			fr->cpu_opts.type = dreidnow;
-			fr->cpu_opts.dct36 = dct36_3dnow; /* 3DNow! optimized dct36() */
-			fr->cpu_opts.synth_1to1 = synth_1to1_3dnow;
-			fr->cpu_opts.dct64 = dct64_i386; /* use the 3dnow one? */
-			done = 1;
-		}
-		#endif
-		#ifdef OPT_MMX
-		if(   !done && (auto_choose || !strcasecmp(cpu, "mmx"))
-		   && cpu_mmx(cpu_flags) )
-		{
-			int go = 1;
-			if(fr->p.force_rate)
-			{
-				#ifdef PENTIUM_FALLBACK
-				if(!auto_choose){ if(NOQUIET) error("I refuse to choose MMX as this will screw up with forced rate!"); }
-				else if(VERBOSE) fprintf(stderr, "Note: Not choosing MMX because flexible rate not supported.\n");
-
-				go = 0;
-				#else
-				error("You will hear some awful sound because of flexible rate being chosen with MMX decoder!");
-				#endif
-			}
-			if(go){ /* temporary hack for flexible rate bug, not going indent this - fix it instead! */
-			chosen = "MMX";
-			fr->cpu_opts.type = mmx;
-			fr->cpu_opts.class = mmxsse;
-			fr->cpu_opts.synth_1to1 = synth_1to1_mmx;
-			fr->cpu_opts.dct64 = dct64_mmx;
-			fr->cpu_opts.make_decode_tables   = make_decode_tables_mmx;
-			fr->cpu_opts.init_layer3_gainpow2 = init_layer3_gainpow2_mmx;
-			fr->cpu_opts.init_layer2_table    = init_layer2_table_mmx;
-			done = 1;
-			}
-		}
-		#endif
-		#ifdef OPT_I586
-		if(!done && (auto_choose || !strcasecmp(cpu, "i586")))
-		{
-			chosen = "i586/pentium";
-			fr->cpu_opts.type = ifuenf;
-			fr->cpu_opts.synth_1to1 = synth_1to1_i586;
-			fr->cpu_opts.synth_1to1_i586_asm = synth_1to1_i586_asm;
-			fr->cpu_opts.dct64 = dct64_i386;
-			done = 1;
-		}
-		#endif
-		#ifdef OPT_I586_DITHER
-		if(!done && (auto_choose || !strcasecmp(cpu, "i586_dither")))
-		{
-			chosen = "dithered i586/pentium";
-			fr->cpu_opts.type = ifuenf_dither;
-			fr->cpu_opts.synth_1to1 = synth_1to1_i586;
-			fr->cpu_opts.dct64 = dct64_i386;
-			fr->cpu_opts.synth_1to1_i586_asm = synth_1to1_i586_asm_dither;
-			done = 1;
-		}
-		#endif
-	}
-	#ifdef OPT_I486 /* that won't cooperate nicely in multi opt mode - forcing i486 in layer3.c */
-	if(!done && (auto_choose || !strcasecmp(cpu, "i486")))
-	{
-		chosen = "i486";
-		fr->cpu_opts.type = ivier;
-		fr->cpu_opts.synth_1to1 = synth_1to1_i386; /* i486 function is special */
-		fr->cpu_opts.dct64 = dct64_i386;
-		done = 1;
-	}
-	#endif
-	#ifdef OPT_I386
-	if(!done && (auto_choose || !strcasecmp(cpu, "i386")))
-	{
-		chosen = "i386";
-		fr->cpu_opts.type = idrei;
-		fr->cpu_opts.synth_1to1 = synth_1to1_i386;
-		fr->cpu_opts.dct64 = dct64_i386;
-		done = 1;
-	}
-	#endif
-
-	if(done) /* set common x86 functions */
-	{
-		fr->cpu_opts.synth_1to1_mono = synth_1to1_mono_i386;
-		fr->cpu_opts.synth_1to1_mono2stereo = synth_1to1_mono2stereo_i386;
-		fr->cpu_opts.synth_1to1_8bit = synth_1to1_8bit_i386;
-		fr->cpu_opts.synth_1to1_8bit_mono = synth_1to1_8bit_mono_i386;
-		fr->cpu_opts.synth_1to1_8bit_mono2stereo = synth_1to1_8bit_mono2stereo_i386;
-	}
-	#endif /* OPT_X86 */
-
-	#ifdef OPT_ALTIVEC
-	if(!done && (auto_choose || !strcasecmp(cpu, "altivec")))
-	{
-		chosen = "AltiVec";
-		fr->cpu_opts.type = altivec;
-		fr->cpu_opts.dct64 = dct64_altivec;
-		fr->cpu_opts.synth_1to1 = synth_1to1_altivec;
-		fr->cpu_opts.synth_1to1_mono = synth_1to1_mono_altivec;
-		fr->cpu_opts.synth_1to1_mono2stereo = synth_1to1_mono2stereo_altivec;
-		fr->cpu_opts.synth_1to1_8bit = synth_1to1_8bit_altivec;
-		fr->cpu_opts.synth_1to1_8bit_mono = synth_1to1_8bit_mono_altivec;
-		fr->cpu_opts.synth_1to1_8bit_mono2stereo = synth_1to1_8bit_mono2stereo_altivec;
-		done = 1;
-	}
-	#endif
-
-	#ifdef OPT_GENERIC
-	if(!done && (auto_choose || !strcasecmp(cpu, "generic")))
-	{
-		chosen = "generic";
-		fr->cpu_opts.type = generic;
-		fr->cpu_opts.dct64 = dct64;
-		fr->cpu_opts.synth_1to1 = synth_1to1;
-		fr->cpu_opts.synth_1to1_mono = synth_1to1_mono;
-		fr->cpu_opts.synth_1to1_mono2stereo = synth_1to1_mono2stereo;
-		fr->cpu_opts.synth_1to1_8bit = synth_1to1_8bit;
-		fr->cpu_opts.synth_1to1_8bit_mono = synth_1to1_8bit_mono;
-		fr->cpu_opts.synth_1to1_8bit_mono2stereo = synth_1to1_8bit_mono2stereo;
-		done = 1;
-	}
-	#endif
-#endif
-	if(done)
-	{
-		if(VERBOSE) fprintf(stderr, "Decoder: %s\n", chosen);
-		return 1;
-	}
-	else
-	{
-		if(NOQUIET) error("Could not set optimization!");
-		return 0;
-	}
-}
-
-enum optdec dectype(const char* decoder)
-{
-	if(decoder == NULL) return autodec;
-	if(!strcasecmp(decoder, "3dnowext"))    return dreidnowext;
-	if(!strcasecmp(decoder, "3dnow"))       return dreidnow;
-	if(!strcasecmp(decoder, "sse"))         return sse;
-	if(!strcasecmp(decoder, "mmx"))         return mmx;
-	if(!strcasecmp(decoder, "generic"))     return generic;
-	if(!strcasecmp(decoder, "altivec"))     return altivec;
-	if(!strcasecmp(decoder, "i386"))        return idrei;
-	if(!strcasecmp(decoder, "i486"))        return ivier;
-	if(!strcasecmp(decoder, "i586"))        return ifuenf;
-	if(!strcasecmp(decoder, "i586_dither")) return ifuenf_dither;
-	return nodec;
-}
-
