@@ -10,6 +10,13 @@
 #include "id3.h"
 #include "debug.h"
 
+#ifndef NO_ID3V2 /* Only the main parsing routine will always be there. */
+
+/* We know the usual text frames plus some specifics. */
+#define KNOWN_FRAMES 4
+static const char frame_type[KNOWN_FRAMES][5] = { "COMM", "TXXX", "RVA2", "USLT" };
+enum frame_types { unknown = -2, text = -1, comment, extra, rva2, uslt };
+
 /* UTF support definitions */
 
 typedef void (*text_converter)(mpg123_string *sb, unsigned char* source, size_t len);
@@ -179,22 +186,22 @@ void id3_link(mpg123_handle *fr)
 */
 void store_id3_text(mpg123_string *sb, char *source, size_t source_size, const int noquiet)
 {
-	int encoding;
+	unsigned int encoding;
 	int bwidth;
 	if(!source_size)
 	{
 		debug("Empty id3 data!");
 		return;
 	}
-	encoding = source[0];
+	encoding = (unsigned int) source[0];
 	++source;
 	--source_size;
-	debug1("encoding: %i", encoding);
+	debug1("encoding: %u", encoding);
 	/* A note: ID3v2.3 uses UCS-2 non-variable 16bit encoding, v2.4 uses UTF16.
 	   UTF-16 uses a reserved/private range in UCS-2 to add the magic, so we just always treat it as UTF. */
 	if(encoding > 3)
 	{
-		if(noquiet) warning1("Unknown text encoding %d, assuming ISO8859-1 - I will probably screw a bit up!", encoding);
+		if(noquiet) warning1("Unknown text encoding %u, assuming ISO8859-1 - I will probably screw a bit up!", encoding);
 		encoding = 0;
 	}
 	bwidth = encoding_widths[encoding];
@@ -208,7 +215,7 @@ void store_id3_text(mpg123_string *sb, char *source, size_t source_size, const i
 	if(source_size % bwidth)
 	{
 		/* When we need two bytes for a character, it's strange to have an uneven bytestream length. */
-		if(noquiet) warning2("Weird tag size %d for encoding %d - I will probably trim too early or something but I think the MP3 is broken.", (int)source_size, encoding);
+		if(noquiet) warning2("Weird tag size %d for encoding %u - I will probably trim too early or something but I think the MP3 is broken.", (int)source_size, encoding);
 		source_size -= source_size % bwidth;
 	}
 	text_converters[encoding](sb, (unsigned char*)source, source_size);
@@ -273,8 +280,9 @@ static void process_text(mpg123_handle *fr, char *realdata, size_t realsize, cha
 	if(VERBOSE4) fprintf(stderr, "Note: ID3v2 %c%c%c%c text frame: %s\n", id[0], id[1], id[2], id[3], t->text.p);
 }
 
-/* Store a new comment that perhaps is a RVA / RVA_ALBUM/AUDIOPHILE / RVA_MIX/RADIO one */
-static void process_comment(mpg123_handle *fr, char *realdata, size_t realsize, int rva_level, char *id)
+/* Store a new comment that perhaps is a RVA / RVA_ALBUM/AUDIOPHILE / RVA_MIX/RADIO one
+   Special gimmik: It also stores USLT to the texts. Stucture is the same as for comments. */
+static void process_comment(mpg123_handle *fr, enum frame_types tt, char *realdata, size_t realsize, int rva_level, char *id)
 {
 	/* Text encoding          $xx */
 	/* Language               $xx xx xx */
@@ -290,7 +298,7 @@ static void process_comment(mpg123_handle *fr, char *realdata, size_t realsize,
 		if(NOQUIET) error1("Invalid frame size of %lu (too small for anything).", (unsigned long)realsize);
 		return;
 	}
-	xcom = add_comment(fr);
+	xcom = (tt == uslt ? add_text(fr) : add_comment(fr));
 	if(VERBOSE4) fprintf(stderr, "Note: Storing comment from %s encoding\n", enc_name(realdata[0]));
 	if(xcom == NULL)
 	{
@@ -315,10 +323,11 @@ static void process_comment(mpg123_handle *fr, char *realdata, size_t realsize,
 
 	if(VERBOSE4)
 	{
-		fprintf(stderr, "Note: ID3 comment desc: %s\n", xcom->description.fill > 0 ? xcom->description.p : "");
-		fprintf(stderr, "Note: ID3 comment text: %s\n", xcom->text.fill > 0 ? xcom->text.p : "");
+		fprintf(stderr, "Note: ID3 comm/uslt desc: %s\n", xcom->description.fill > 0 ? xcom->description.p : "");
+		fprintf(stderr, "Note: ID3 comm/uslt text: %s\n", xcom->text.fill > 0 ? xcom->text.p : "");
 	}
-	if(xcom->description.fill > 0 && xcom->text.fill > 0)
+	/* Look out for RVA info only when we really deal with a straight comment. */
+	if(tt == comment && xcom->description.fill > 0 && xcom->text.fill > 0)
 	{
 		int rva_mode = -1; /* mix / album */
 		if(   !strcasecmp(xcom->description.p, "rva")
@@ -447,6 +456,8 @@ int promote_framename(mpg123_handle *fr, char *id) /* fr because of VERBOSE macr
 	return -1;
 }
 
+#endif /* NO_ID3V2 */
+
 /*
 	trying to parse ID3v2.3 and ID3v2.4 tags...
 
@@ -515,14 +526,17 @@ int parse_new_id3(mpg123_handle *fr, unsigned long first4bytes)
 		return 0;
 	}
 	debug1("ID3v2: tag data length %lu", length);
+#ifndef NO_ID3V2
 	if(VERBOSE2) fprintf(stderr,"Note: ID3v2.%i rev %i tag of %lu bytes\n", major, buf[0], length);
 	/* skip if unknown version/scary flags, parse otherwise */
 	if((flags & UNKNOWN_FLAGS) || (major > 4) || (major < 2))
 	{
 		/* going to skip because there are unknown flags set */
 		if(NOQUIET) warning2("ID3v2: Won't parse the ID3v2 tag with major version %u and flags 0x%xu - some extra code may be needed", major, flags);
+#endif
 		if((ret2 = fr->rd->skip_bytes(fr,length)) < 0) /* will not store data in backbuff! */
 		ret = ret2;
+#ifndef NO_ID3V2
 	}
 	else
 	{
@@ -560,9 +574,7 @@ int parse_new_id3(mpg123_handle *fr, unsigned long first4bytes)
 						int head_part = fr->id3v2.version == 2 ? 3 : 4; /* bytes of frame title and of framesize value */
 						/* level 1,2,3 - 0 is info from lame/info tag! */
 						/* rva tags with ascending significance, then general frames */
-						#define KNOWN_FRAMES 3
-						const char frame_type[KNOWN_FRAMES][5] = { "COMM", "TXXX", "RVA2" }; /* plus all text frames... */
-						enum { unknown = -2, text = -1, comment, extra, rva2 } tt = unknown;
+						enum frame_types tt = unknown;
 						/* we may have entered the padding zone or any other strangeness: check if we have valid frame id characters */
 						for(i=0; i< head_part; ++i)
 						if( !( ((tagdata[tagpos+i] > 47) && (tagdata[tagpos+i] < 58))
@@ -664,7 +676,8 @@ int parse_new_id3(mpg123_handle *fr, unsigned long first4bytes)
 								switch(tt)
 								{
 									case comment:
-										process_comment(fr, (char*)realdata, realsize, comment+1, id);
+									case uslt:
+										process_comment(fr, tt, (char*)realdata, realsize, comment+1, id);
 									break;
 									case extra: /* perhaps foobar2000's work */
 										process_extra(fr, (char*)realdata, realsize, extra+1, id);
@@ -742,6 +755,7 @@ tagparse_cleanup:
 			else ret = 0;
 		}
 	}
+#endif /* NO_ID3V2 */
 	/* skip footer if present */
 	if((ret > 0) && (flags & FOOTER_FLAG) && ((ret2 = fr->rd->skip_bytes(fr,length)) < 0)) ret = ret2;
 
@@ -753,6 +767,8 @@ tagparse_cleanup:
 	#undef UNKOWN_FLAGS
 }
 
+#ifndef NO_ID3V2 /* Disabling all the rest... */
+
 static void convert_latin1(mpg123_string *sb, unsigned char* s, size_t l)
 {
 	size_t length = l;
@@ -886,3 +902,5 @@ static void convert_utf8(mpg123_string *sb, unsigned char* source, size_t len)
 	}
 	else mpg123_free_string(sb);
 }
+
+#endif
