@@ -1,7 +1,7 @@
 /*
 	libmpg123: MPEG Audio Decoder library
 
-	copyright 1995-2008 by the mpg123 project - free software under the terms of the LGPL 2.1
+	copyright 1995-2009 by the mpg123 project - free software under the terms of the LGPL 2.1
 	see COPYING and AUTHORS files in distribution or http://mpg123.org
 
 */
@@ -57,7 +57,6 @@ static int initialized = 0;
 
 #endif
 
-
 #ifdef GAPLESS
 /*
 	Take the buffer after a frame decode (strictly: it is the data from frame fr->num!) and cut samples out.
@@ -105,11 +104,13 @@ int attribute_align_arg mpg123_init(void)
 	ALIGNCHECKK
 	if((sizeof(short) != 2) || (sizeof(long) < 4)) return MPG123_BAD_TYPES;
 
-	init_layer2(); /* inits also shared tables with layer1 */
-	init_layer3();
-#ifndef OPT_MMX_ONLY
-	prepare_decode_tables();
+#ifndef NO_LAYER12
+	init_layer12(); /* inits also shared tables with layer1 */
 #endif
+#ifndef NO_LAYER3
+	init_layer3();
+#endif
+	prepare_decode_tables();
 	check_decoders();
 	initialized = 1;
 	return MPG123_OK;
@@ -132,7 +133,7 @@ mpg123_handle attribute_align_arg *mpg123_parnew(mpg123_pars *mp, const char* de
 {
 	mpg123_handle *fr = NULL;
 	int err = MPG123_OK;
-#ifdef DEBUG
+#if (defined CCALIGN) && (defined NEED_ALIGNCHECK) && ((defined DEBUG) || (defined CHECK_ALIGN))
 #ifdef CCALIGN
 	double ALIGNED(16) altest[4];
 	if(((size_t)altest) % 16 != 0)
@@ -159,7 +160,8 @@ mpg123_handle attribute_align_arg *mpg123_parnew(mpg123_pars *mp, const char* de
 	}
 	if(fr != NULL)
 	{
-		if((frame_outbuffer(fr) != 0) || (frame_buffers(fr) != 0))
+		/* Cleanup that mess! ... use mpg123_decoder / decode_update! */
+		if(frame_outbuffer(fr) != 0)
 		{
 			err = MPG123_NO_BUFFERS;
 			frame_exit(fr);
@@ -168,11 +170,16 @@ mpg123_handle attribute_align_arg *mpg123_parnew(mpg123_pars *mp, const char* de
 		}
 		else
 		{
-			opt_make_decode_tables(fr);
+			/* I smell cleanup here... with get_next_frame() */
+/*			if(decode_update(fr) != 0)
+			{
+				err = fr->err != MPG123_OK ? fr->err : MPG123_BAD_DECODER;
+				frame_exit(fr);
+				free(fr);
+				fr = NULL;
+			}
+			else */
 			fr->decoder_change = 1;
-			/* happening on frame change instead:
-			init_layer3_stuff(fr);
-			init_layer2_stuff(fr); */
 		}
 	}
 	else if(err == MPG123_OK) err = MPG123_OUT_OF_MEM;
@@ -205,13 +212,14 @@ int attribute_align_arg mpg123_decoder(mpg123_handle *mh, const char* decoder)
 		return MPG123_ERR;
 	}
 	/* New buffers for decoder are created in frame_buffers() */
-	if((frame_outbuffer(mh) != 0) || (frame_buffers(mh) != 0))
+	if((frame_outbuffer(mh) != 0))
 	{
 		mh->err = MPG123_NO_BUFFERS;
 		frame_exit(mh);
 		return MPG123_ERR;
 	}
-	opt_make_decode_tables(mh);
+	/* I smell cleanup here... with get_next_frame() */
+	decode_update(mh);
 	mh->decoder_change = 1;
 	return MPG123_OK;
 }
@@ -249,9 +257,6 @@ int attribute_align_arg mpg123_par(mpg123_pars *mp, enum mpg123_parms key, long
 		case MPG123_FLAGS:
 #ifndef GAPLESS
 			if(val & MPG123_GAPLESS) ret = MPG123_NO_GAPLESS;
-#endif
-#ifdef FLOATOUT
-			if(val & MPG123_FORCE_8BIT) ret = MPG123_NO_8BIT;
 #endif
 			if(ret == MPG123_OK) mp->flags = val;
 			debug1("set flags to 0x%lx", (unsigned long) mp->flags);
@@ -270,12 +275,21 @@ int attribute_align_arg mpg123_par(mpg123_pars *mp, enum mpg123_parms key, long
 			debug1("set flags to 0x%lx", (unsigned long) mp->flags);
 		break;
 		case MPG123_FORCE_RATE: /* should this trigger something? */
+#ifdef NO_NTOM
+			if(val > 0)
+			ret = MPG123_BAD_RATE;
+#else
 			if(val > 96000) ret = MPG123_BAD_RATE;
 			else mp->force_rate = val < 0 ? 0 : val; /* >0 means enable, 0 disable */
+#endif
 		break;
 		case MPG123_DOWN_SAMPLE:
+#ifdef NO_DOWNSAMPLE
+			if(val != 0) ret = MPG123_BAD_RATE;
+#else
 			if(val < 0 || val > 2) ret = MPG123_BAD_RATE;
 			else mp->down_sample = (int)val;
+#endif
 		break;
 		case MPG123_RVA:
 			if(val < 0 || val > MPG123_RVA_MAX) ret = MPG123_BAD_RVA;
@@ -288,14 +302,16 @@ int attribute_align_arg mpg123_par(mpg123_pars *mp, enum mpg123_parms key, long
 			mp->doublespeed = val < 0 ? 0 : val;
 		break;
 		case MPG123_ICY_INTERVAL:
+#ifndef NO_ICY
 			mp->icy_interval = val > 0 ? val : 0;
+#else
+			if(val > 0) ret = MPG123_BAD_PARAM;
+#endif
 		break;
 		case MPG123_OUTSCALE:
-#ifdef FLOATOUT
-			mp->outscale = fval;
-#else
-			mp->outscale = val;
-#endif
+			/* Choose the value that is non-zero, if any.
+			   Downscaling integers to 1.0 . */
+			mp->outscale = val == 0 ? fval : (double)val/SHORT_SCALE;
 		break;
 		case MPG123_TIMEOUT:
 #ifndef WIN32
@@ -345,7 +361,12 @@ int attribute_align_arg mpg123_getpar(mpg123_pars *mp, enum mpg123_parms key, lo
 			if(val) *val = mp->flags;
 		break;
 		case MPG123_FORCE_RATE:
-			if(val) *val = mp->force_rate;
+			if(val) 
+#ifdef NO_NTOM
+			*val = 0;
+#else
+			*val = mp->force_rate;
+#endif
 		break;
 		case MPG123_DOWN_SAMPLE:
 			if(val) *val = mp->down_sample;
@@ -360,18 +381,27 @@ int attribute_align_arg mpg123_getpar(mpg123_pars *mp, enum mpg123_parms key, lo
 			if(val) *val = mp->doublespeed;
 		break;
 		case MPG123_ICY_INTERVAL:
+#ifndef NO_ICY
 			if(val) *val = (long)mp->icy_interval;
+#else
+			if(val) *val = 0;
+#endif
 		break;
 		case MPG123_OUTSCALE:
-#ifdef FLOATOUT
 			if(fval) *fval = mp->outscale;
-#else
-			if(val) *val = mp->outscale;
-#endif
+			if(val) *val = (long)(mp->outscale*SHORT_SCALE);
 		break;
 		case MPG123_RESYNC_LIMIT:
 			if(val) *val = mp->resync_limit;
 		break;
+		case MPG123_INDEX_SIZE:
+			if(val)
+#ifdef FRAME_INDEX
+			*val = mp->index_size;
+#else
+			*val = 0; /* graceful fallback: no index is index of zero size */
+#endif
+		break;
 		default:
 			ret = MPG123_BAD_PARAM;
 	}
@@ -490,9 +520,16 @@ int attribute_align_arg mpg123_replace_reader( mpg123_handle *mh,
 int decode_update(mpg123_handle *mh)
 {
 	long native_rate;
+	int b;
 	ALIGNCHECK(mh);
 	native_rate = frame_freq(mh);
-	debug2("updating decoder structure with native rate %li and af.rate %li", native_rate, mh->af.rate);
+
+	b = frame_output_format(mh); /* Select the new output format based on given constraints. */
+	if(b < 0) return MPG123_ERR;
+
+	if(b == 1) mh->new_format = 1; /* Store for later... */
+
+	debug3("updating decoder structure with native rate %li and af.rate %li (new format: %i)", native_rate, mh->af.rate, mh->new_format);
 	if(mh->af.rate == native_rate) mh->down_sample = 0;
 	else if(mh->af.rate == native_rate>>1) mh->down_sample = 1;
 	else if(mh->af.rate == native_rate>>2) mh->down_sample = 2;
@@ -504,8 +541,9 @@ int decode_update(mpg123_handle *mh)
 		case 2:
 			mh->down_sample_sblimit = SBLIMIT>>(mh->down_sample);
 			/* With downsampling I get less samples per frame */
-			mh->outblock = sizeof(sample_t)*mh->af.channels*(spf(mh)>>mh->down_sample);
+			mh->outblock = samples_to_bytes(mh, (spf(mh)>>mh->down_sample));
 		break;
+#ifndef NO_NTOM
 		case 3:
 		{
 			if(synth_ntom_set_step(mh) != 0) return -1;
@@ -515,12 +553,13 @@ int decode_update(mpg123_handle *mh)
 				mh->down_sample_sblimit /= frame_freq(mh);
 			}
 			else mh->down_sample_sblimit = SBLIMIT;
-			mh->outblock = sizeof(sample_t) * mh->af.channels *
+			mh->outblock = mh->af.encsize * mh->af.channels *
 			               ( ( NTOM_MUL-1+spf(mh)
 			                   * (((size_t)NTOM_MUL*mh->af.rate)/frame_freq(mh))
 			                 )/NTOM_MUL );
 		}
 		break;
+#endif
 	}
 
 	if(!(mh->p.flags & MPG123_FORCE_MONO))
@@ -530,8 +569,7 @@ int decode_update(mpg123_handle *mh)
 	}
 	else mh->single = (mh->p.flags & MPG123_FORCE_MONO)-1;
 	if(set_synth_functions(mh) != 0) return -1;;
-	init_layer3_stuff(mh);
-	init_layer2_stuff(mh);
+
 	do_rva(mh);
 	debug3("done updating decoder structure with native rate %li and af.rate %li and down_sample %i", frame_freq(mh), mh->af.rate, mh->down_sample);
 
@@ -540,7 +578,8 @@ int decode_update(mpg123_handle *mh)
 
 size_t attribute_align_arg mpg123_safe_buffer()
 {
-	return sizeof(sample_t)*2*1152*NTOM_MAX;
+	/* real is the largest possible output (it's 32bit float, 32bit int or 64bit double). */
+	return sizeof(real)*2*1152*NTOM_MAX;
 }
 
 size_t attribute_align_arg mpg123_outblock(mpg123_handle *mh)
@@ -561,9 +600,10 @@ static int get_next_frame(mpg123_handle *mh)
 			debug1("ignoring frame %li", (long)mh->num);
 			/* Decoder structure must be current! decode_update has been called before... */
 			(mh->do_layer)(mh); mh->buffer.fill = 0;
+#ifndef NO_NTOM
 			/* The ignored decoding may have failed. Make sure ntom stays consistent. */
 			if(mh->down_sample == 3) ntom_set_ntom(mh, mh->num+1);
-
+#endif
 			mh->to_ignore = mh->to_decode = FALSE;
 		}
 		/* Read new frame data; possibly breaking out here for MPG123_NEED_MORE. */
@@ -588,7 +628,20 @@ static int get_next_frame(mpg123_handle *mh)
 			debug("big header change");
 			change = 1;
 		}
-	} while(mh->num < mh->firstframe);
+		/* Now some accounting: Look at the numbers and decide if we want this frame. */
+		++mh->playnum;
+		/* Plain skipping without decoding, only when frame is not ignored on next cycle. */
+		if(mh->num < mh->firstframe || (mh->p.doublespeed && (mh->playnum % mh->p.doublespeed)))
+		{
+			if(!(mh->to_ignore && mh->num < mh->firstframe && mh->num >= mh->ignoreframe))
+			{
+				frame_skip(mh);
+				/* Should one fix NtoM here or not? NtoM with seeking is not repeatable sample accurate... */
+			}
+		}
+		/* Or, we are finally done and have a new frame. */
+		else break;
+	} while(1);
 	/* When we start actually using the CRC, this could move into the loop... */
 	/* A question of semantics ... should I fold start_frame and frame_number into firstframe/lastframe? */
 	if(mh->lastframe >= 0 && mh->num > mh->lastframe)
@@ -598,15 +651,16 @@ static int get_next_frame(mpg123_handle *mh)
 	}
 	if(change)
 	{
-		int b = frame_output_format(mh); /* Select the new output format based on given constraints. */
-		if(b < 0) return MPG123_ERR; /* not nice to fail here... perhaps once should add possibility to repeat this step */
-		if(decode_update(mh) < 0) return MPG123_ERR; /* dito... */
+		if(decode_update(mh) < 0)  /* dito... */
+		return MPG123_ERR;
+
+debug1("new format: %i", mh->new_format);
+
 		mh->decoder_change = 0;
-		if(b == 1) mh->new_format = 1; /* Store for later... */
 #ifdef GAPLESS
 		if(mh->fresh)
 		{
-			b=0;
+			int b=0;
 			/* Prepare offsets for gapless decoding. */
 			debug1("preparing gapless stuff with native rate %li", frame_freq(mh));
 			frame_gapless_realinit(mh);
@@ -628,19 +682,76 @@ static int get_next_frame(mpg123_handle *mh)
 void decode_the_frame(mpg123_handle *fr)
 {
 	size_t needed_bytes = samples_to_bytes(fr, frame_outs(fr, fr->num+1)-frame_outs(fr, fr->num));
-
 	fr->clip += (fr->do_layer)(fr);
-	/* There could be less data than promised. */
-	if(fr->buffer.fill < needed_bytes)
+	/*fprintf(stderr, "frame %"OFF_P": got %"SIZE_P" / %"SIZE_P"\n", fr->num,(size_p)fr->buffer.fill, (size_p)needed_bytes);*/
+	/* There could be less data than promised.
+	   Also, then debugging, we look out for coding errors that could result in _more_ data than expected. */
+#ifdef DEBUG
+	if(fr->buffer.fill != needed_bytes)
 	{
-		if(NOQUIET) fprintf(stderr, "Note: broken frame %li, filling up with zeroes\n", (long)fr->num);
+#endif
+		if(fr->buffer.fill < needed_bytes)
+		{
+			if(VERBOSE2)
+			fprintf(stderr, "Note: broken frame %li, filling up with %"SIZE_P" zeroes, from %"SIZE_P"\n", (long)fr->num, (size_p)(needed_bytes-fr->buffer.fill), (size_p)fr->buffer.fill);
 
-		/* One could do a loop with individual samples instead... but zero is zero. */
-		memset(fr->buffer.data + fr->buffer.fill, 0, needed_bytes - fr->buffer.fill);
-		fr->buffer.fill = needed_bytes;
-		/* ntom_val will be wrong when the decoding wasn't carried out completely */
-		if(fr->down_sample == 3) ntom_set_ntom(fr, fr->num+1);
+			/* One could do a loop with individual samples instead... but zero is zero. */
+			memset(fr->buffer.data + fr->buffer.fill, 0, needed_bytes - fr->buffer.fill);
+			fr->buffer.fill = needed_bytes;
+#ifndef NO_NTOM
+			/* ntom_val will be wrong when the decoding wasn't carried out completely */
+			ntom_set_ntom(fr, fr->num+1);
+#endif
+		}
+#ifdef DEBUG
+		else
+		{
+			if(NOQUIET)
+			error2("I got _more_ bytes than expected (%"SIZE_P" / %"SIZE_P"), that should not be possible!", (size_p)fr->buffer.fill, (size_p)needed_bytes);
+		}
 	}
+#endif
+	/* Handle unsigned output formats via reshifting after decode here. */
+#ifndef NO_32BIT
+	if(fr->af.encoding == MPG123_ENC_UNSIGNED_32)
+	{ /* 32bit signed -> unsigned */
+		size_t i;
+		int32_t *ssamples;
+		uint32_t *usamples;
+		ssamples = (int32_t*)fr->buffer.data;
+		usamples = (uint32_t*)fr->buffer.data;
+		debug("converting output to unsigned 32 bit integer");
+		for(i=0; i<fr->buffer.fill/sizeof(int32_t); ++i)
+		{
+			/* Different strategy since we don't have a larger type at hand.
+				 Also watch out for silly +-1 fun because integer constants are signed in C90! */
+			if(ssamples[i] >= 0)
+			usamples[i] = (uint32_t)ssamples[i] + 2147483647+1;
+			/* The smalles value goes zero. */
+			else if(ssamples[i] == ((int32_t)-2147483647-1))
+			usamples[i] = 0;
+			/* Now -value is in the positive range of signed int ... so it's a possible value at all. */
+			else
+			usamples[i] = (uint32_t)2147483647+1 - (uint32_t)(-ssamples[i]);
+		}
+	}
+#endif
+#ifndef NO_16BIT
+	if(fr->af.encoding == MPG123_ENC_UNSIGNED_16)
+	{
+		size_t i;
+		short *ssamples;
+		unsigned short *usamples;
+		ssamples = (short*)fr->buffer.data;
+		usamples = (unsigned short*)fr->buffer.data;
+		debug("converting output to unsigned 16 bit integer");
+		for(i=0; i<fr->buffer.fill/sizeof(short); ++i)
+		{
+			long tmp = (long)ssamples[i]+32768;
+			usamples[i] = (unsigned short)tmp;
+		}
+	}
+#endif
 }
 
 /*
@@ -669,7 +780,7 @@ int attribute_align_arg mpg123_decode_frame(mpg123_handle *mh, off_t *num, unsig
 		{
 			if(mh->new_format)
 			{
-				mh->new_format = 0;
+				debug("notifiying new format");
 				return MPG123_NEW_FORMAT;
 			}
 			if(num != NULL) *num = mh->num;
@@ -758,7 +869,7 @@ int attribute_align_arg mpg123_decode(mpg123_handle *mh, const unsigned char *in
 		{
 			if(mh->new_format)
 			{
-				mh->new_format = 0;
+				debug("notifiying new format");
 				return MPG123_NEW_FORMAT;
 			}
 			if(mh->buffer.size - mh->buffer.fill < mh->outblock)
@@ -918,16 +1029,20 @@ static int do_the_seek(mpg123_handle *mh)
 		mh->to_decode = FALSE;
 		return MPG123_OK;
 	}
+#ifndef NO_NTOM
 	if(mh->down_sample == 3)
 	{
 		ntom_set_ntom(mh, fnum);
-		debug3("fixed ntom for frame %"OFF_P" to %i, num=%"OFF_P, fnum, mh->ntom_val[0], mh->num);
+		debug3("fixed ntom for frame %"OFF_P" to %lu, num=%"OFF_P, fnum, mh->ntom_val[0], mh->num);
 	}
+#endif
 	b = mh->rd->seek_frame(mh, fnum);
 	debug1("seek_frame returned: %i", b);
 	if(b<0) return b;
 	/* Only mh->to_ignore is TRUE. */
 	if(mh->num < mh->firstframe) mh->to_decode = FALSE;
+
+	mh->playnum = mh->num;
 	return 0;
 }
 
@@ -1089,12 +1204,13 @@ off_t attribute_align_arg mpg123_length(mpg123_handle *mh)
 	if(b<0) return b;
 	if(mh->track_samples > -1) length = mh->track_samples;
 	else if(mh->track_frames > 0) length = mh->track_frames*spf(mh);
-	else if(mh->rdat.filelen >= 0) /* Let the case of 0 length just fall through. */
+	else if(mh->rdat.filelen > 0) /* Let the case of 0 length just fall through. */
 	{
 		/* A bad estimate. Ignoring tags 'n stuff. */
 		double bpf = mh->mean_framesize ? mh->mean_framesize : compute_bpf(mh);
 		length = (off_t)((double)(mh->rdat.filelen)/bpf*spf(mh));
 	}
+	else if(mh->rdat.filelen == 0) return mpg123_tell(mh); /* we could be in feeder mode */
 	else return MPG123_ERR; /* No length info there! */
 
 	length = frame_ins2outs(mh, length);
@@ -1158,7 +1274,13 @@ int attribute_align_arg mpg123_id3(mpg123_handle *mh, mpg123_id3v1 **v1, mpg123_
 	{
 		id3_link(mh);
 		if(v1 != NULL && mh->rdat.flags & READER_ID3TAG) *v1 = (mpg123_id3v1*) mh->id3buf;
-		if(v2 != NULL) *v2 = &mh->id3v2;
+		if(v2 != NULL)
+#ifdef NO_ID3V2
+		*v2 = NULL;
+#else
+		*v2 = &mh->id3v2;
+#endif
+
 		mh->metaflags |= MPG123_ID3;
 		mh->metaflags &= ~MPG123_NEW_ID3;
 	}
@@ -1169,6 +1291,7 @@ int attribute_align_arg mpg123_icy(mpg123_handle *mh, char **icy_meta)
 {
 	ALIGNCHECK(mh);
 	if(mh == NULL) return MPG123_ERR;
+#ifndef NO_ICY
 	if(icy_meta == NULL)
 	{
 		mh->err = MPG123_NULL_POINTER;
@@ -1183,11 +1306,19 @@ int attribute_align_arg mpg123_icy(mpg123_handle *mh, char **icy_meta)
 		mh->metaflags &= ~MPG123_NEW_ICY;
 	}
 	return MPG123_OK;
+#else
+	mh->err = MPG123_MISSING_FEATURE;
+	return MPG123_ERR;
+#endif
 }
 
 char* attribute_align_arg mpg123_icy2utf8(const char* icy_text)
 {
+#ifndef NO_ICY
 	return icy2utf8(icy_text);
+#else
+	return NULL;
+#endif
 }
 
 
@@ -1275,7 +1406,9 @@ static const char *mpg123_error[] =
 	"Inappropriate NULL-pointer provided.",
 	"Bad key value given.",
 	"There is no frame index (disabled in this build).",
-	"Frame index operation failed."
+	"Frame index operation failed.",
+	"Decoder setup failed (invalid combination of settings?)",
+	"Feature not in this build."
 };
 
 const char* attribute_align_arg mpg123_plain_strerror(int errcode)
