@@ -16,6 +16,7 @@
 /* ABI conformance for other compilers.
    mpg123 needs 16byte-aligned stack for SSE and friends.
    gcc provides that, but others don't necessarily. */
+#ifdef ABI_ALIGN_FUN
 #ifndef attribute_align_arg
 #if defined(__GNUC__) && (__GNUC__ > 4 || __GNUC__ == 4 && __GNUC_MINOR__>1)
 #    define attribute_align_arg __attribute__((force_align_arg_pointer))
@@ -25,6 +26,10 @@
 #    define NEED_ALIGNCHECK /* Other compilers get code to catch misaligned stack. */
 #endif
 #endif
+#else
+#define attribute_align_arg
+/* We won't try the align check... */
+#endif
 
 /* export DLL symbols */
 #if defined(WIN32) && defined(DYNAMIC_BUILD)
@@ -57,6 +62,14 @@
 #  define REAL_SCANF "%Lf"
 #  define REAL_PRINTF "%Lf"
 #elif defined(REAL_IS_FIXED)
+/* Disable some output formats for fixed point decoder... */
+#ifndef NO_REAL
+#define NO_REAL
+#endif
+#ifndef NO_32BIT
+#define NO_32BIT
+#endif
+
 # define real long
 
 # define REAL_RADIX            15
@@ -79,14 +92,20 @@
 #  define REAL_PRINTF "%f"
 #endif
 
+#ifndef REAL_IS_FIXED
+# if (defined SIZEOF_INT32_T) && (SIZEOF_INT32_T != 4)
+#  error "Bad 32bit types!!!"
+# endif
+#endif
+
 #ifndef DOUBLE_TO_REAL
-# define DOUBLE_TO_REAL(x)     (x)
+# define DOUBLE_TO_REAL(x)     (real)(x)
 #endif
 #ifndef REAL_TO_DOUBLE
 # define REAL_TO_DOUBLE(x)     (x)
 #endif
 #ifndef REAL_TO_SHORT
-# define REAL_TO_SHORT(x)      (x)
+# define REAL_TO_SHORT(x)      (short)(x)
 #endif
 #ifndef REAL_PLUS_32767
 # define REAL_PLUS_32767       32767.0
@@ -94,6 +113,12 @@
 #ifndef REAL_MINUS_32768
 # define REAL_MINUS_32768      -32768.0
 #endif
+#ifndef REAL_PLUS_S32
+# define REAL_PLUS_S32 2147483647.0
+#endif
+#ifndef REAL_MINUS_S32
+# define REAL_MINUS_S32 -2147483648.0
+#endif
 #ifndef REAL_MUL
 # define REAL_MUL(x, y)                ((x) * (y))
 #endif
@@ -115,22 +140,19 @@
 #define         MPG_MD_DUAL_CHANNEL     2
 #define         MPG_MD_MONO             3
 
-/* float output only for generic decoder! */
-#ifdef FLOATOUT
-#define MAXOUTBURST 1.0
-#define scale_t double
-#else
-/* I suspect that 32767 would be a better idea here, but Michael put this in... */
-#define MAXOUTBURST 32768
-#define scale_t long
-#endif
+/* We support short or float output samples...
+   Short integer amplitude is scaled by this. */
+#define SHORT_SCALE 32768
+/* That scales a short-scaled value to a 32bit integer scaled one
+   value = 2**31/2**15 */
+#define S32_RESCALE 65536
 
 /* Pre Shift fo 16 to 8 bit converter table */
 #define AUSHIFT (3)
 
+#include "optimize.h"
 #include "decode.h"
 #include "parse.h"
-#include "optimize.h"
 #include "frame.h"
 
 /* fr is a mpg123_handle* by convention here... */
@@ -142,5 +164,8 @@
 #define PVERB(mp, level) (!((mp)->flags & MPG123_QUIET) && (mp)->verbose >= (level))
 
 int decode_update(mpg123_handle *mh);
+/* residing in format.c  */
+off_t samples_to_bytes(mpg123_handle *fr , off_t s);
+off_t bytes_to_samples(mpg123_handle *fr , off_t b);
 
 #endif
