@@ -39,6 +39,7 @@ static off_t   posix_lseek(int fd, off_t offset, int whence){ return lseek(fd, o
 
 static ssize_t plain_fullread(mpg123_handle *fr,unsigned char *buf, ssize_t count);
 
+#ifndef NO_FEEDER
 /* Bufferchain methods. */
 static void bc_init(struct bufferchain *bc);
 static void bc_reset(struct bufferchain *bc);
@@ -51,6 +52,10 @@ static ssize_t bc_give(struct bufferchain *bc, unsigned char *out, size_t size);
 static ssize_t bc_skip(struct bufferchain *bc, ssize_t count);
 static ssize_t bc_seekback(struct bufferchain *bc, ssize_t count);
 static void bc_forget(struct bufferchain *bc);
+#else
+#define bc_init(a)
+#define bc_reset(a)
+#endif
 
 /* A normal read and a read with timeout. */
 static ssize_t plain_read(mpg123_handle *fr, void *buf, size_t count)
@@ -84,6 +89,7 @@ static ssize_t timeout_read(mpg123_handle *fr, void *buf, size_t count)
 }
 #endif
 
+#ifndef NO_ICY
 /* stream based operation  with icy meta data*/
 static ssize_t icy_fullread(mpg123_handle *fr, unsigned char *buf, ssize_t count)
 {
@@ -184,6 +190,9 @@ static ssize_t icy_fullread(mpg123_handle *fr, unsigned char *buf, ssize_t count
 	/* debug1("done reading, got %li", (long)cnt); */
 	return cnt;
 }
+#else
+#define icy_fullread NULL
+#endif /* NO_ICY */
 
 /* stream based operation */
 static ssize_t plain_fullread(mpg123_handle *fr,unsigned char *buf, ssize_t count)
@@ -390,6 +399,7 @@ static off_t get_fileinfo(mpg123_handle *fr)
 	return len;
 }
 
+#ifndef NO_FEEDER
 /* Methods for the buffer chain, mainly used for feed reader, but not just that. */
 
 static void bc_init(struct bufferchain *bc)
@@ -687,8 +697,18 @@ static ssize_t buffered_fullread(mpg123_handle *fr, unsigned char *out, ssize_t
 	if(gotcount != count){ if(NOQUIET) error("gotcount != count"); return READER_ERROR; }
 	else return gotcount;
 }
-
-
+#else
+int feed_more(mpg123_handle *fr, const unsigned char *in, long count)
+{
+	fr->err = MPG123_MISSING_FEATURE;
+	return -1;
+}
+off_t feed_set_pos(mpg123_handle *fr, off_t pos)
+{
+	fr->err = MPG123_MISSING_FEATURE;
+	return -1;
+}
+#endif /* NO_FEEDER */
 
 /*****************************************************************
  * read frame helper
@@ -743,6 +763,15 @@ struct reader readers[] =
 		stream_rewind,
 		NULL
 	},
+#ifdef NO_FEEDER
+#define feed_init NULL
+#define feed_read NULL
+#define buffered_fullread NULL
+#define buffered_seek_frame NULL
+#define feed_back_bytes NULL
+#define feed_skip_bytes NULL
+#define buffered_forget NULL
+#endif
 	{ /* READER_FEED */
 		feed_init,
 		stream_close,
@@ -824,7 +853,13 @@ static int default_init(mpg123_handle *fr)
 #ifndef WIN32
 	if(fr->p.timeout > 0)
 	{
-		int flags = fcntl(fr->rdat.filept, F_GETFL);
+		int flags;
+		if(fr->rdat.r_read != NULL)
+		{
+			error("Timeout reading does not work with user-provided read function. Implement it yourself!");
+			return -1;
+		}
+		flags = fcntl(fr->rdat.filept, F_GETFL);
 		flags |= O_NONBLOCK;
 		fcntl(fr->rdat.filept, F_SETFL, flags);
 		fr->rdat.fdread = timeout_read;
@@ -851,16 +886,23 @@ static int default_init(mpg123_handle *fr)
 	/* Switch reader to a buffered one, if allowed. */
 	else if(fr->p.flags & MPG123_SEEKBUFFER)
 	{
+#ifdef NO_FEEDER
+		error("Buffered readers not supported in this build.");
+		fr->err = MPG123_MISSING_FEATURE;
+		return -1;
+#else
 		if     (fr->rd == &readers[READER_STREAM])
 		{
 			fr->rd = &readers[READER_BUF_STREAM];
 			fr->rdat.fullread = plain_fullread;
 		}
+#ifndef NO_ICY
 		else if(fr->rd == &readers[READER_ICY_STREAM])
 		{
 			fr->rd = &readers[READER_BUF_ICY_STREAM];
 			fr->rdat.fullread = icy_fullread;
 		}
+#endif
 		else
 		{
 			if(NOQUIET) error("mpg123 Programmer's fault: invalid reader");
@@ -869,6 +911,7 @@ static int default_init(mpg123_handle *fr)
 		bc_init(&fr->rdat.buffer);
 		fr->rdat.filelen = 0; /* We carry the offset, but never know how big the stream is. */
 		fr->rdat.flags |= READER_BUFFERED;
+#endif /* NO_FEEDER */
 	}
 	return 0;
 }
@@ -876,7 +919,9 @@ static int default_init(mpg123_handle *fr)
 
 void open_bad(mpg123_handle *mh)
 {
+#ifndef NO_ICY
 	clear_icy(&mh->icy);
+#endif
 	mh->rd = &bad_reader;
 	mh->rdat.flags = 0;
 	bc_init(&mh->rdat.buffer);
@@ -885,11 +930,25 @@ void open_bad(mpg123_handle *mh)
 int open_feed(mpg123_handle *fr)
 {
 	debug("feed reader");
+#ifdef NO_FEEDER
+	error("Buffered readers not supported in this build.");
+	fr->err = MPG123_MISSING_FEATURE;
+	return -1;
+#else
+#ifndef NO_ICY
+	if(fr->p.icy_interval > 0)
+	{
+		if(NOQUIET) error("Feed reader cannot do ICY parsing!");
+
+		return -1;
+	}
 	clear_icy(&fr->icy);
+#endif
 	fr->rd = &readers[READER_FEED];
 	fr->rdat.flags = 0;
 	if(fr->rd->init(fr) < 0) return -1;
 	return 0;
+#endif /* NO_FEEDER */
 }
 
 int open_stream(mpg123_handle *fr, const char *bs_filenam, int fd)
@@ -919,6 +978,7 @@ int open_stream(mpg123_handle *fr, const char *bs_filenam, int fd)
 	fr->rdat.flags = 0;
 	if(filept_opened)	fr->rdat.flags |= READER_FD_OPENED;
 
+#ifndef NO_ICY
 	if(fr->p.icy_interval > 0)
 	{
 		debug("ICY reader");
@@ -927,6 +987,7 @@ int open_stream(mpg123_handle *fr, const char *bs_filenam, int fd)
 		fr->rd = &readers[READER_ICY_STREAM];
 	}
 	else
+#endif
 	{
 		fr->rd = &readers[READER_STREAM];
 		debug("stream reader");
