@@ -1,7 +1,7 @@
 /*
 	mpg123: main code of the program (not of the decoder...)
 
-	copyright 1995-2008 by the mpg123 project - free software under the terms of the LGPL 2.1
+	copyright 1995-2009 by the mpg123 project - free software under the terms of the LGPL 2.1
 	see COPYING and AUTHORS files in distribution or http://mpg123.org
 	initially written by Michael Hipp
 */
@@ -110,6 +110,8 @@ struct parameter param = {
 	,0 /* keep_open */
 	,0 /* force_utf8 */
 	,INDEX_SIZE
+	,NULL /* force_encoding */
+	,1. /* preload */
 };
 
 mpg123_handle *mh = NULL;
@@ -356,17 +358,14 @@ topt opts[] = {
 /*	{'g', "gain",        GLO_ARG | GLO_LONG, 0, &ao.gain,    0}, FIXME */
 	{'r', "rate",        GLO_ARG | GLO_LONG, 0, &param.force_rate,  0},
 	{0,   "8bit",        GLO_INT,  set_frameflag, &frameflag, MPG123_FORCE_8BIT},
+	{0,   "float",       GLO_INT,  set_frameflag, &frameflag, MPG123_FORCE_FLOAT},
 	{0,   "headphones",  0,                  set_output_h, 0,0},
 	{0,   "speaker",     0,                  set_output_s, 0,0},
 	{0,   "lineout",     0,                  set_output_l, 0,0},
 	{'o', "output",      GLO_ARG | GLO_CHAR, set_output, 0,  0},
 	{0,   "list-modules",0,        list_modules, NULL,  0}, 
 	{'a', "audiodevice", GLO_ARG | GLO_CHAR, 0, &param.output_device,  0},
-#ifdef FLOATOUT
-	{'f', "scale",       GLO_ARG | GLO_DOUBLE, 0, &param.outscale,   0},
-#else
 	{'f', "scale",       GLO_ARG | GLO_LONG, 0, &param.outscale,   0},
-#endif
 	{'n', "frames",      GLO_ARG | GLO_LONG, 0, &param.frame_number,  0},
 	#ifdef HAVE_TERMIOS
 	{'C', "control",     GLO_INT,  0, &param.term_ctrl, TRUE},
@@ -440,6 +439,8 @@ topt opts[] = {
 	{0, "fuzzy", GLO_INT,  set_frameflag, &frameflag, MPG123_FUZZY},
 	{0, "index-size", GLO_ARG|GLO_LONG, 0, &param.index_size, 0},
 	{0, "no-seekbuffer", GLO_INT, unset_frameflag, &frameflag, MPG123_SEEKBUFFER},
+	{'e', "encoding", GLO_ARG|GLO_CHAR, 0, &param.force_encoding, 0},
+	{0, "preload", GLO_ARG|GLO_DOUBLE, 0, &param.preload, 0},
 	{0, 0, 0, 0, 0, 0}
 };
 
@@ -717,12 +718,9 @@ int main(int argc, char *argv[])
 	mpg123_getpar(mp, MPG123_RVA, &param.rva, NULL);
 	mpg123_getpar(mp, MPG123_DOWNSPEED, &param.halfspeed, NULL);
 	mpg123_getpar(mp, MPG123_UPSPEED, &param.doublespeed, NULL);
-#ifdef FLOATOUT
-	mpg123_getpar(mp, MPG123_OUTSCALE, NULL, &param.outscale);
-#else
 	mpg123_getpar(mp, MPG123_OUTSCALE, &param.outscale, NULL);
-#endif
 	mpg123_getpar(mp, MPG123_FLAGS, &parr, NULL);
+	mpg123_getpar(mp, MPG123_INDEX_SIZE, &param.index_size, NULL);
 	param.flags = (int) parr;
 	param.flags |= MPG123_SEEKBUFFER; /* Default on, for HTTP streams. */
 	mpg123_getpar(mp, MPG123_RESYNC_LIMIT, &param.resync_limit, NULL);
@@ -749,7 +747,7 @@ int main(int argc, char *argv[])
 
 	if(param.list_cpu)
 	{
-		char **all_dec = mpg123_decoders();
+		const char **all_dec = mpg123_decoders();
 		printf("Builtin decoders:");
 		while(*all_dec != NULL){ printf(" %s", *all_dec); ++all_dec; }
 		printf("\n");
@@ -758,7 +756,7 @@ int main(int argc, char *argv[])
 	}
 	if(param.test_cpu)
 	{
-		char **all_dec = mpg123_supported_decoders();
+		const char **all_dec = mpg123_supported_decoders();
 		printf("Supported decoders:");
 		while(*all_dec != NULL){ printf(" %s", *all_dec); ++all_dec; }
 		printf("\n");
@@ -822,13 +820,8 @@ int main(int argc, char *argv[])
 	    && ++libpar
 	    && MPG123_OK == (result = mpg123_par(mp, MPG123_TIMEOUT, param.timeout, 0))
 #endif
-#ifdef FLOATOUT
-	    && ++libpar
-	    && MPG123_OK == (result = mpg123_par(mp, MPG123_OUTSCALE, 0, param.outscale))
-#else
 	    && ++libpar
 	    && MPG123_OK == (result = mpg123_par(mp, MPG123_OUTSCALE, param.outscale, 0))
-#endif
 			))
 	{
 		error2("Cannot set library parameter %i: %s", libpar, mpg123_plain_strerror(result));
@@ -836,8 +829,12 @@ int main(int argc, char *argv[])
 	}
 	if (!(param.listentry < 0) && !param.quiet) print_title(stderr); /* do not pollute stdout! */
 
-	if( (result = mpg123_par(mp, MPG123_INDEX_SIZE, param.index_size, 0.)) != MPG123_OK )
-	error1("Setting of frame index size failed: %s", mpg123_plain_strerror(result));
+	{
+		long default_index;
+		mpg123_getpar(mp, MPG123_INDEX_SIZE, &default_index, NULL);
+		if( param.index_size != default_index && (result = mpg123_par(mp, MPG123_INDEX_SIZE, param.index_size, 0.)) != MPG123_OK )
+		error1("Setting of frame index size failed: %s", mpg123_plain_strerror(result));
+	}
 
 	if(param.force_rate && param.down_sample)
 	{
@@ -947,7 +944,7 @@ int main(int argc, char *argv[])
 			{
 				fprintf(stderr, "%s", "So, you have trouble getting an output format... this is the matrix of currently possible formats:\n");
 				print_capabilities(ao, mh);
-				fprintf(stderr, "%s", "Somehow the input data and your choices don't allow one of these.\nTried enabling resampling with a forced rate (only 2to1 and 4to1 downsampling is tried automatically)?\nTried allowing mono/stereo?\n");
+				fprintf(stderr, "%s", "Somehow the input data and your choices don't allow one of these.\n");
 			}
 			mpg123_close(mh);
 			continue;
@@ -1077,7 +1074,7 @@ static void usage(int err)  /* print syntax & exit */
 	fprintf(o,"   -w <filename> write Output as WAV file\n");
 	fprintf(o,"   -k n  skip first n frames [0]        -n n  decode only n frames [all]\n");
 	fprintf(o,"   -c    check range violations         -y    DISABLE resync on errors\n");
-	fprintf(o,"   -b n  output buffer: n Kbytes [0]    -f n  change scalefactor [%g]\n", (double)param.outscale);
+	fprintf(o,"   -b n  output buffer: n Kbytes [0]    -f n  change scalefactor [%li]\n", param.outscale);
 	fprintf(o,"   -r n  set/force samplerate [auto]    -g n  set audio hardware output gain\n");
 	fprintf(o,"   -os,-ol,-oh  output to built-in speaker,line-out connector,headphones\n");
 	#ifdef NAS
@@ -1111,6 +1108,7 @@ static void want_usage(char* arg)
 
 static void long_usage(int err)
 {
+	char *enclist;
 	FILE* o = stdout;
 	if(err)
 	{
@@ -1162,7 +1160,7 @@ static void long_usage(int err)
 	fprintf(o,"        --no-3dnow         force use of floating-pointer routine (obsoleted by --cpu)\n");
 	#endif
 	fprintf(o," -g     --gain             set audio hardware output gain\n");
-	fprintf(o," -f <n> --scale <n>        scale output samples (soft gain, default=%g)\n", (double)param.outscale);
+	fprintf(o," -f <n> --scale <n>        scale output samples (soft gain - based on 32768), default=%li)\n", param.outscale);
 	fprintf(o,"        --rva-mix,\n");
 	fprintf(o,"        --rva-radio        use RVA2/ReplayGain values for mix/radio mode\n");
 	fprintf(o,"        --rva-album,\n");
@@ -1176,6 +1174,9 @@ static void long_usage(int err)
 	fprintf(o," -4     --4to1             4:1 downsampling\n");
   fprintf(o,"        --pitch <value>    set hardware pitch (speedup/down, 0 is neutral; 0.05 is 5%%)\n");
 	fprintf(o,"        --8bit             force 8 bit output\n");
+	fprintf(o,"        --float            force floating point output (internal precision)\n");
+	audio_enclist(&enclist);
+	fprintf(o," -e <c> --encoding <c>     force a specific encoding c (%s)\n", enclist != NULL ? enclist : "OOM!");
 	fprintf(o," -d n   --doublespeed n    play only every nth frame\n");
 	fprintf(o," -h n   --halfspeed   n    play every frame n times\n");
 	fprintf(o,"        --equalizer        exp.: scales freq. bands acrd. to 'equalizer.dat'\n");
@@ -1188,6 +1189,7 @@ static void long_usage(int err)
 	fprintf(o," -o l   --lineout          (aix/hp/sun) output to lineout\n");
 #ifndef NOXFERMEM
 	fprintf(o," -b <n> --buffer <n>       set play buffer (\"output cache\")\n");
+	fprintf(o,"        --preload <value>  fraction of buffer to fill before playback\n");
 	fprintf(o,"        --smooth           keep buffer over track boundaries\n");
 #endif
 
