@@ -1,7 +1,7 @@
 /*
 	mpg123: main code of the program (not of the decoder...)
 
-	copyright 1995-2008 by the mpg123 project - free software under the terms of the LGPL 2.1
+	copyright 1995-2009 by the mpg123 project - free software under the terms of the LGPL 2.1
 	see COPYING and AUTHORS files in distribution or http://mpg123.org
 	initially written by Michael Hipp
 
@@ -84,11 +84,7 @@ struct parameter
 	long doublespeed;
 	long start_frame;  /* frame offset to begin with */
 	long frame_number; /* number of frames to decode */
-#ifdef FLOATOUT
-	double outscale;
-#else
 	long outscale;
-#endif
 	int flags;
 	long force_rate;
 	int talk_icy;
@@ -100,6 +96,8 @@ struct parameter
 	int keep_open; /* Whether to keep files open after end reached, for remote control mode, perhaps terminal control, too. */
 	int force_utf8; /* Regardless of environment, always print out verbatim UTF for metadata. */
 	long index_size; /* size of frame index */
+	char *force_encoding;
+	double preload; /* buffer preload size (fraction of full buffer) */
 };
 
 extern char *equalfile;
