@@ -382,7 +382,7 @@ static void term_handle_input(mpg123_handle *fr, audio_output_t *ao, int do_dela
 	case RVA_KEY:
 		if(++param.rva > MPG123_RVA_MAX) param.rva = 0;
 		mpg123_param(fr, MPG123_RVA, param.rva, 0);
-		mpg123_volume(fr, -1);
+		mpg123_volume_change(fr, 0.);
 	break;
 	case PREV_KEY:
 		if(!param.usebuffer) ao->flush(ao);
@@ -421,13 +421,26 @@ static void term_handle_input(mpg123_handle *fr, audio_output_t *ao, int do_dela
 	}
 	break;
 	case FRAME_INDEX_KEY:
-		print_index(fr);
+	case VARIOUS_INFO_KEY:
+		if(param.verbose) fprintf(stderr, "\n");
+		switch(val) /* because of tolower() ... */
 		{
-			long accurate;
-			if(mpg123_getstate(fr, MPG123_ACCURATE, &accurate, NULL) == MPG123_OK)
-			fprintf(stderr, "Accurate position: %s\n", (accurate == 0 ? "no" : "yes"));
-			else
-			error1("Unable to get state: %s", mpg123_strerror(fr));
+			case FRAME_INDEX_KEY:
+			print_index(fr);
+			{
+				long accurate;
+				if(mpg123_getstate(fr, MPG123_ACCURATE, &accurate, NULL) == MPG123_OK)
+				fprintf(stderr, "Accurate position: %s\n", (accurate == 0 ? "no" : "yes"));
+				else
+				error1("Unable to get state: %s", mpg123_strerror(fr));
+			}
+			break;
+			case VARIOUS_INFO_KEY:
+			{
+				const char* curdec = mpg123_current_decoder(fr);
+				if(curdec == NULL) fprintf(stderr, "Cannot get decoder info!\n");
+				else fprintf(stderr, "Active decoder: %s\n", curdec);
+			}
 		}
 	break;
 	default:
